print("import: 'tensorboardX'")
import tensorboardX

print("import: 'tensorboardX.crc32c'")
import tensorboardX.crc32c

print("import: 'tensorboardX.embedding'")
import tensorboardX.embedding

print("import: 'tensorboardX.event_file_writer'")
import tensorboardX.event_file_writer

print("import: 'tensorboardX.onnx_graph'")
import tensorboardX.onnx_graph

print("import: 'tensorboardX.record_writer'")
import tensorboardX.record_writer

print("import: 'tensorboardX.summary'")
import tensorboardX.summary

print("import: 'tensorboardX.writer'")
import tensorboardX.writer

print("import: 'tensorboardX.x2num'")
import tensorboardX.x2num

